@@ -103,7 +103,15 @@ struct MVMStorageSpec {
 #define MVM_STORAGE_SPEC_CAN_BOX_STR    4
 #define MVM_STORAGE_SPEC_CAN_BOX_MASK   7
 
-/* Flags that may be set on any collectable. */
+/* Flags that may be set on any collectable.
+ * These are split across two bytes so that we can set MVM_CF_HAS_OBJECT_ID and
+ * MVM_CF_REF_FROM_GEN2 from different threads without a data race.
+ * (Plenty of work for the CPU cache coherency hardware, but no race.)
+ * This avoids the need to use atomic operations to manipulate the flags. */
+
+/* Needed to provide a migration path for the code in Rakudo's perl6_ops.c */
+#define MVM_COLLECTABLE_FLAGS1
+
 typedef enum {
     /* Is a type object (and thus not a concrete instance). */
     MVM_CF_TYPE_OBJECT = 1,
@@ -111,49 +119,50 @@ typedef enum {
     /* Is an STable. */
     MVM_CF_STABLE = 2,
 
-    /* Is a heap-promoted call frame. */
+    /* Is a heap-promoted call frame.
+     * Beware, this flag bit must not overlap with any values set by macros
+     * MVM_FRAME_FLAG_* in frame.h */
     MVM_CF_FRAME = 4,
 
-    /* Has already been seen once in GC nursery. */
-    MVM_CF_NURSERY_SEEN = 8,
-
-    /* Has been promoted to the old generation. */
-    MVM_CF_SECOND_GEN = 16,
-
-    /* Is shared - that is, more than one thread knows about it. */
-    MVM_CF_SHARED = 32,
-
-    /* Has already been added to the gen2 aggregates pointing to nursery
-     * objects list. */
-    MVM_CF_IN_GEN2_ROOT_LIST = 64,
-
-    /* A full GC run has found this object to be live. */
-    MVM_CF_GEN2_LIVE = 128,
-
-    /* This object in fromspace is live with a valid forwarder. */
-    /* TODO - should be possible to use the same bit for this and GEN2_LIVE. */
-    MVM_CF_FORWARDER_VALID = 256,
-
     /* Have we allocated memory to store a serialization index? */
-    MVM_CF_SERIALZATION_INDEX_ALLOCATED = 512,
+    MVM_CF_SERIALZATION_INDEX_ALLOCATED = 8,
 
     /* Have we arranged a persistent object ID for this object? */
-    MVM_CF_HAS_OBJECT_ID = 1024,
+    MVM_CF_HAS_OBJECT_ID = 16,
 
     /* Have we flagged this object as something we must never repossess? */
     /* Note: if you're hunting for a flag, some day in the future when we
      * have used them all, this one is easy enough to eliminate by having the
      * tiny number of objects marked this way in a remembered set. */
-    MVM_CF_NEVER_REPOSSESS = 2048,
+    MVM_CF_NEVER_REPOSSESS = 32
+} MVMCollectableFlags1;
 
-    /* Has this item been chained into a gen2 freelist? This is only used in
-     * GC debug more. */
-    MVM_CF_DEBUG_IN_GEN2_FREE_LIST = 4096,
+typedef enum {
+    /* Has already been seen once in GC nursery. */
+    MVM_CF_NURSERY_SEEN = 1,
+
+    /* Has been promoted to the old generation. */
+    MVM_CF_SECOND_GEN = 2,
+
+    /* Has already been added to the gen2 aggregates pointing to nursery
+     * objects list. */
+    MVM_CF_IN_GEN2_ROOT_LIST = 4,
+
+    /* A full GC run has found this object to be live. */
+    MVM_CF_GEN2_LIVE = 8,
+
+    /* This object in fromspace is live with a valid forwarder. */
+    /* TODO - should be possible to use the same bit for this and GEN2_LIVE. */
+    MVM_CF_FORWARDER_VALID = 16,
 
     /* Is this object a nursery object that has been referenced from gen2?
      * Used to promote it earlier. */
-    MVM_CF_REF_FROM_GEN2 = 8192,
-} MVMCollectableFlags;
+    MVM_CF_REF_FROM_GEN2 = 32,
+
+    /* Has this item been chained into a gen2 freelist? This is only used in
+     * GC debug more. */
+    MVM_CF_DEBUG_IN_GEN2_FREE_LIST = 64
+} MVMCollectableFlags2;
 
 #ifdef MVM_USE_OVERFLOW_SERIALIZATION_INDEX
 struct MVMSerializationIndex {
@@ -197,14 +206,13 @@ struct MVMCollectable {
         MVMSTable *st;
     } sc_forward_u;
 
-    /* Identifier of the thread that currently owns the object, if any. If the
-     * object is unshared, then this is always the creating thread. If it is
-     * shared then it's whoever currently holds the mutex on it, or 0 if there
-     * is no held mutex. */
+    /* Identifier of the thread that created the object. 0 if this is a
+     * non-heap frame. */
     MVMuint32 owner;
 
     /* Collectable flags (see MVMCollectableFlags). */
-    MVMuint16 flags;
+    MVMuint8 flags1;
+    MVMuint8 flags2;
 
     /* Object size, in bytes. */
     MVMuint16 size;
@@ -371,6 +379,10 @@ struct MVMSTable {
     /* If this STable is currently in the process of being repossessed. Used
      * to trigger clearup of memory pre-repossession. */
     MVMuint8 being_repossessed;
+
+    /* If this STable represents a type that can be the target of a
+     * change_type - that is to say, it's been mixed in to. */
+    MVMuint8 is_mixin_type;
 };
 
 /* The representation operations table. Note that representations are not
@@ -689,7 +701,7 @@ struct MVMREPROps {
 #define OBJECT_BODY(o)   (&(((MVMObjectStooge *)(o))->data))
 
 /* Macros for getting/setting type-objectness. */
-#define IS_CONCRETE(o)   (!(((MVMObject *)o)->header.flags & MVM_CF_TYPE_OBJECT))
+#define IS_CONCRETE(o)   (!(((MVMObject *)o)->header.flags1 & MVM_CF_TYPE_OBJECT))
 
 /* Some functions related to 6model core functionality. */
 MVM_PUBLIC MVMObject * MVM_6model_get_how(MVMThreadContext *tc, MVMSTable *st);
@@ -710,7 +722,7 @@ MVMuint64 MVM_6model_next_type_cache_id(MVMThreadContext *tc);
 void MVM_6model_never_repossess(MVMThreadContext *tc, MVMObject *obj);
 
 MVM_STATIC_INLINE char *MVM_6model_get_debug_name(MVMThreadContext *tc, MVMObject *obj) {
-    return STABLE(obj)->debug_name ? STABLE(obj)->debug_name : "";
+    return obj ? STABLE(obj)->debug_name ? STABLE(obj)->debug_name : "" : "";
 }
 MVM_STATIC_INLINE char *MVM_6model_get_stable_debug_name(MVMThreadContext *tc, MVMSTable *stable) {
     return stable->debug_name ? stable->debug_name : "";
