@@ -7,15 +7,16 @@
  */
 
 /* Version information. */
-#define MVM_VERSION "2018.12"
-#define MVM_VERSION_MAJOR 2018
-#define MVM_VERSION_MINOR 12
+#define MVM_VERSION "2021.02"
+#define MVM_VERSION_MAJOR 2021
+#define MVM_VERSION_MINOR 02
 #define MVM_VERSION_PATCH 0
 
 /* Various compiler-specific pragmii. */
 #define MVM_NO_RETURN 
 #define MVM_NO_RETURN_ATTRIBUTE __attribute__((noreturn))
 #define MVM_FORMAT(X, Y, Z) __attribute__((format(X, Y, Z)))
+#define MVM_FALLTHROUGH 
 
 /* DLL interface definitions. */
 #define MVM_DLL_IMPORT 
@@ -33,9 +34,23 @@
 #define MVM_HAS_PTHREAD_YIELD 0
 #endif
 
+/* pthread_setname_np() detection */
+#if 0
+#define MVM_HAS_PTHREAD_SETNAME_NP 0
+#endif
+
+#if 0
+#define MVM_HAS_FN_MALLOC_TRIM 0
+#endif
+
 /* How this compiler does static inline functions. */
 #define MVM_STATIC_INLINE static __inline__
 
+#if 1
+/* How this compiler declares thread local storage. */
+#define MVM_THREAD_LOCAL _Thread_local
+#endif
+
 #if 0
 #define MVM_CAN_UNALIGNED_INT32
 #endif
@@ -50,6 +65,9 @@
 
 #define MVM_PTR_SIZE 4
 
+#if 0
+#define MVM_HAS_SUBSTANDARD_POW
+#endif
 
 #if 1
 #define MVM_BOOL _Bool
