@@ -28,10 +28,13 @@ MVM_STATIC_INLINE void * MVM_realloc(void *p, size_t size) {
 MVM_STATIC_INLINE void * MVM_recalloc(void *p, size_t old_size, size_t size) {
     void *ptr = realloc(p, size);
 
-    if (!ptr && size > 0)
-        MVM_panic_allocation_failed(size);
+    if (size > 0) {
+        if (!ptr)
+            MVM_panic_allocation_failed(size);
 
-    memset((char *)ptr + old_size, 0, size - old_size);
+        if (size > old_size)
+            memset((char *)ptr + old_size, 0, size - old_size);
+    }
 
     return ptr;
 }
@@ -40,9 +43,8 @@ MVM_STATIC_INLINE void MVM_free(void *p) {
     free(p);
 }
 
-
 #define MVM_free_null(addr) do { \
-    MVM_free((void *)(addr)); \
+    MVM_free((addr)); \
     (addr) = NULL; \
 } while (0)
 
