@@ -33,6 +33,7 @@ typedef struct {
 /* Various common string constants. */
 struct MVMStringConsts {
     MVMString *empty;
+    MVMString *Int;
     MVMString *Str;
     MVMString *Num;
     MVMString *integer;
@@ -68,6 +69,7 @@ struct MVMStringConsts {
     MVMString *buf_type;
     MVMString *write;
     MVMString *stdin_fd;
+    MVMString *stdin_fd_close;
     MVMString *stdout_fd;
     MVMString *stderr_fd;
     MVMString *nativeref;
@@ -78,6 +80,8 @@ struct MVMStringConsts {
     MVMString *ready;
     MVMString *multidim;
     MVMString *entry_point;
+    MVMString *resolve_lib_name;
+    MVMString *resolve_lib_name_arg;
     MVMString *kind;
     MVMString *instrumented;
     MVMString *heap;
@@ -86,32 +90,24 @@ struct MVMStringConsts {
     MVMString *path;
     MVMString *config;
     MVMString *replacement;
+    MVMString *dot;
 };
 
-/* An entry in the representations registry. */
-struct MVMReprRegistry {
-    /* name of the REPR */
-    MVMString *name;
+struct MVMEventSubscriptions {
+    uv_mutex_t mutex_event_subscription;
 
-    /* the REPR vtable */
-    const MVMREPROps *repr;
+    MVMObject *subscription_queue;
 
-    /* the uthash hash handle inline struct. */
-    UT_hash_handle hash_handle;
+    MVMObject *GCEvent;
+    MVMObject *SpeshOverviewEvent;
+
+    MVMuint64 vm_startup_hrtime;
+    MVMnum64  vm_startup_now;
 };
 
-/* An entry in the persistent object IDs hash, used to give still-movable
- * objects a lifetime-unique ID. */
-struct MVMObjectId {
-    /* The current object address. */
-    MVMObject *current;
-
-    /* Then gen2 address that forms the persistent ID, and where we'll move
-     * the object to if it lives long enough. */
-    MVMCollectable *gen2_addr;
-
-    /* Hash handle. */
-    UT_hash_handle hash_handle;
+struct MVMSerializationContextWeakHashEntry {
+    struct MVMStrHashHandle hash_handle;
+    MVMSerializationContextBody *scb;
 };
 
 /* Represents a MoarVM instance. */
@@ -175,6 +171,11 @@ struct MVMInstance {
      * join in, but this lets them wait efficieintly. */
     uv_cond_t cond_blocked_can_continue;
 
+    /* Whether the coordinator considers the run to be fully completed,
+     * including cleanup of exited threads. */
+    AO_t gc_completed;
+    uv_cond_t cond_gc_completed;
+
     /* The number of threads that have yet to acknowledge the finish. */
     AO_t gc_ack;
 
@@ -199,7 +200,7 @@ struct MVMInstance {
 
     /* Persistent object ID hash, used to give nursery objects a lifetime
      * unique ID. Plus a lock to protect it. */
-    MVMObjectId *object_ids;
+    MVMPtrHashTable     object_ids;
     uv_mutex_t    mutex_object_ids;
 
     /* Fixed size allocator. */
@@ -217,17 +218,20 @@ struct MVMInstance {
     /* Number of representations registered so far. */
     MVMuint32 num_reprs;
 
-    /* An array mapping representation IDs to registry entries. */
-    MVMReprRegistry **repr_list;
+    /* An array mapping representation IDs to REPR vtables. */
+    const MVMREPROps **repr_vtables;
 
-    /* A hash mapping representation names to registry entries. */
-    MVMReprRegistry *repr_hash;
+    /* An array mapping representation IDs to REPR names. */
+    MVMString **repr_names;
+
+    /* A hash mapping representation names to IDs. */
+    MVMIndexHashTable repr_hash;
 
     /* Mutex for REPR registration. */
     uv_mutex_t mutex_repr_registry;
 
     /* Container type registry and mutex to protect it. */
-    MVMContainerRegistry *container_registry;
+    MVMStrHashTable       container_registry;
     uv_mutex_t      mutex_container_registry;
 
     /* Hash of all known serialization contexts. Marked for GC iff
@@ -236,7 +240,7 @@ struct MVMInstance {
      * simply nulled. That makes it a small memory leak if a lot of
      * SCs are created and go away over time. The mutex protects all
      * the weakhash and all SCs list. */
-    MVMSerializationContextBody  *sc_weakhash;
+    MVMStrHashTable               sc_weakhash;
     MVMSerializationContextBody **all_scs;
     MVMuint32                     all_scs_next_idx;
     MVMuint32                     all_scs_alloc;
@@ -246,6 +250,12 @@ struct MVMInstance {
      * than per STable, as this doesn't happen often. */
     uv_mutex_t mutex_parameterization_add;
 
+    /************************************************************************
+     * Configuration programs (for spesh, profiler, ...)
+     ************************************************************************/
+
+    MVMConfigurationProgram *confprog;
+
     /************************************************************************
      * Specializer (dynamic optimization)
      ************************************************************************/
@@ -258,6 +268,7 @@ struct MVMInstance {
     MVMint8 spesh_inline_enabled;
     MVMint8 spesh_inline_log;
     MVMint8 spesh_osr_enabled;
+    MVMint8 spesh_pea_enabled;
     MVMint8 spesh_nodelay;
     MVMint8 spesh_blocking;
 
@@ -335,6 +346,7 @@ struct MVMInstance {
     MVMObject        *event_loop_permit_queue;
     MVMObject        *event_loop_cancel_queue;
     MVMObject        *event_loop_active;
+    MVMObject        *event_loop_free_indices;
     uv_async_t       *event_loop_wakeup;
 
     /* Standard file handles. */
@@ -424,6 +436,10 @@ struct MVMInstance {
     MVMObject *SpeshLog;
     MVMObject *StaticFrameSpesh;
 
+    MVMObject *SpeshPluginState;
+
+    MVMObject *SpeshCandidate;
+
     /* Set of bootstrapping types. */
     MVMBootTypes boot_types;
 
@@ -443,8 +459,8 @@ struct MVMInstance {
     /* Hashes of HLLConfig objects. compiler_hll_configs is those for the
      * running compiler, and the default. compilee_hll_configs is used if
      * hll_compilee_depth is > 0. */
-    MVMHLLConfig *compiler_hll_configs;
-    MVMHLLConfig *compilee_hll_configs;
+    MVMFixKeyHashTable compiler_hll_configs;
+    MVMFixKeyHashTable compilee_hll_configs;
     MVMint64      hll_compilee_depth;
     uv_mutex_t    mutex_hllconfigs;
 
@@ -457,20 +473,20 @@ struct MVMInstance {
     uv_mutex_t    mutex_compiler_registry;
 
     /* Hash of filenames of compunits loaded from disk. */
-    MVMLoadedCompUnitName *loaded_compunits;
+    MVMFixKeyHashTable     loaded_compunits;
     uv_mutex_t       mutex_loaded_compunits;
 
     /* Hash of all loaded DLLs. */
-    MVMDLLRegistry  *dll_registry;
-    uv_mutex_t mutex_dll_registry;
+    MVMFixKeyHashTable     dll_registry;
+    uv_mutex_t       mutex_dll_registry;
 
     /* Hash of all loaded extensions. */
-    MVMExtRegistry  *ext_registry;
-    uv_mutex_t mutex_ext_registry;
+    MVMFixKeyHashTable     ext_registry;
+    uv_mutex_t       mutex_ext_registry;
 
     /* Hash of all registered extension ops. */
-    MVMExtOpRegistry *extop_registry;
-    uv_mutex_t  mutex_extop_registry;
+    MVMFixKeyHashTable     extop_registry;
+    uv_mutex_t       mutex_extop_registry;
 
     /************************************************************************
      * Bytecode instrumentations (profiler, coverage, etc.)
@@ -500,6 +516,9 @@ struct MVMInstance {
     FILE *coverage_log_fh;
     MVMuint32  coverage_control;
 
+    /* The time it takes to run the profiler instrumentation. */
+    MVMuint64 profiling_overhead;
+
     /************************************************************************
      * Debugging
      ************************************************************************/
@@ -518,4 +537,10 @@ struct MVMInstance {
     /* Hash Secrets which is used as the hash seed. This is to avoid denial of
      * service type attacks. */
     MVMuint64 hashSecrets[2];
+
+    /************************************************************************
+     * VM Event subscription
+     ************************************************************************/
+
+    MVMEventSubscriptions subscriptions;
 };
