@@ -1,3 +1,9 @@
+#define MVMGCSTATUS_MASK 3
+#define MVMSUSPENDSTATUS_MASK 12
+
+#define MVM_NUM_TEMP_BIGINTS 3
+#include "tommath.h"
+
 /* Possible values for the thread execution interrupt flag. */
 typedef enum {
     /* Indicates that the thread is currently executing, and should
@@ -19,9 +25,43 @@ typedef enum {
      * run was triggered and the scanning work was stolen. A thread
      * that becomes unblocked upon seeing this will wait for the GC
      * run to be done. */
-    MVMGCStatus_STOLEN = 3
+    MVMGCStatus_STOLEN = 3,
 } MVMGCStatus;
 
+typedef enum {
+    /* Indicates the thread shall continue executing. */
+    MVMSuspendState_NONE = 0,
+
+    /* Indicates the thread shall suspend execution as soon as practical. */
+    MVMSuspendState_SUSPEND_REQUEST = 4,
+
+    /* Indicates the thread has suspended execution and is waiting for
+     * a signal to resume execution. */
+    MVMSuspendState_SUSPENDED = 12,
+} MVMSuspendStatus;
+
+typedef enum {
+    /* Just pass by any line number annotation */
+    MVMDebugSteppingMode_NONE = 0,
+
+    /* Step Over:
+     *  Line annotation: If the line number doesn't match, but the frame does.
+     *  Return from Frame: If the frame matches. */
+    MVMDebugSteppingMode_STEP_OVER = 1,
+
+    /* Step Into:
+     *  Line annotation: If the line number doesn't match in the same frame,
+     *    if the frame doesn't match.
+     *  Return from Frame: If the frame matches. */
+    MVMDebugSteppingMode_STEP_INTO = 2,
+
+    /* Step Out:
+     *  Line annotation: -
+     *  Return from Frame: If the frame matches. */
+    MVMDebugSteppingMode_STEP_OUT = 3,
+} MVMDebugSteppingMode;
+
+
 /* Information associated with an executing thread. */
 struct MVMThreadContext {
     /************************************************************************
@@ -176,6 +216,10 @@ struct MVMThreadContext {
     MVMint32 current_frame_nr;
     MVMint32 next_frame_nr;
 
+    /* JIT return address pointer, so we can figure out the current position in
+     * the code */
+    void **jit_return_address;
+
     /* This thread's current spesh log to write in to, if there curently is
      * one. */
     MVMSpeshLog *spesh_log;
@@ -188,10 +232,16 @@ struct MVMThreadContext {
     /* The spesh stack simulation, perserved between processing logs. */
     MVMSpeshSimStack *spesh_sim_stack;
 
+    /* The current spesh graph that we are optimizing, retained here so we
+     * can GC mark it and so be able to GC at certain points during the
+     * optimization process, giving less GC latency. */
+    MVMSpeshGraph *spesh_active_graph;
+
     /* We try to do better at OSR by creating a fresh log when we enter a new
      * compilation unit. However, for things that EVAL or do a ton of BEGIN,
-     * this does more harm than good. Use this to throttle it back. */
-    MVMuint32 num_compunit_extra_logs;
+     * we risk high memory use. Use this to throttle it by limiting the number
+     * of such extra logs that might exist at a time. */
+    AO_t num_compunit_extra_logs;
 
     /* The current specialization correlation ID, used in logging. */
     MVMuint32 spesh_cid;
@@ -209,24 +259,24 @@ struct MVMThreadContext {
     MVMint32 osr_hunt_frame_nr;
     MVMint32 osr_hunt_num_spesh_candidates;
 
+    /* If we are currently in a spesh plugin, the current set of guards we
+     * have recorded. */
+    MVMSpeshPluginGuard *plugin_guards;
+    MVMObject *plugin_guard_args;
+    MVMuint32 num_plugin_guards;
+
+    MVMSpeshPluginGuard *temp_plugin_guards;
+    MVMObject *temp_plugin_guard_args;
+    MVMuint32 temp_num_plugin_guards;
+
     /************************************************************************
      * Per-thread state held by assorted VM subsystems
      ************************************************************************/
 
-    /* libuv event loop */
-    uv_loop_t *loop;
-
     /* Mutex that must be released if we throw an exception. Used in places
      * like I/O, which grab a mutex but may throw an exception. */
     uv_mutex_t *ex_release_mutex;
 
-    /* Memory buffer pointing to the last thing we serialized, intended to go
-     * into the next compilation unit we write. Also the serialized string
-     * heap, which will be used to seed the compilation unit string heap. */
-    MVMint32      serialized_size;
-    char         *serialized;
-    MVMObject    *serialized_string_heap;
-
     /* Serialization context write barrier disabled depth (anything non-zero
      * means disabled). */
     MVMint32           sc_wb_disable_depth;
@@ -240,12 +290,20 @@ struct MVMThreadContext {
     MVMObject     *cur_dispatcher;
     MVMObject     *cur_dispatcher_for;
 
+    /* Dispatcher to return control to when current dispatcher exhausts. */
+    MVMObject     *next_dispatcher;
+    MVMObject     *next_dispatcher_for;
+
     /* Cache of native code callback data. */
-    MVMNativeCallbackCacheHead *native_callback_cache;
+    MVMStrHashTable native_callback_cache;
 
     /* Random number generator state. */
     MVMuint64 rand_state[2];
 
+    /* Temporary big integers for when we need to upgrade operands in order
+     * to perform an operation. */
+    mp_int *temp_bigints[MVM_NUM_TEMP_BIGINTS];
+
     /* NFA evaluator memory cache, to avoid many allocations; see NFA.c. */
     MVMuint32 *nfa_done;
     MVMuint32 *nfa_curst;
@@ -262,10 +320,61 @@ struct MVMThreadContext {
 
     /* Profiling data collected for this thread, if profiling is on. */
     MVMProfileThreadData *prof_data;
+
+    /* Debug server stepping mode and settings */
+    MVMDebugSteppingMode step_mode;
+    MVMFrame *step_mode_frame;
+    MVMuint32 step_mode_file_idx;
+    MVMuint32 step_mode_line_no;
+    MVMuint64 step_message_id;
+
+    /* Whether the debugserver could request an invocation here.
+     * Requires the current op to be marked invokish and do the
+     * necessary things to cur_op and such. */
+    MVMuint8  debugserver_can_invoke_here;
+
+    MVMuint32 cur_file_idx;
+    MVMuint32 cur_line_no;
+
+    int nested_interpreter;
 };
 
 MVMThreadContext * MVM_tc_create(MVMThreadContext *parent, MVMInstance *instance);
 void MVM_tc_destroy(MVMThreadContext *tc);
 void MVM_tc_set_ex_release_mutex(MVMThreadContext *tc, uv_mutex_t *mutex);
+void MVM_tc_set_ex_release_atomic(MVMThreadContext *tc, AO_t *flag);
 void MVM_tc_release_ex_release_mutex(MVMThreadContext *tc);
 void MVM_tc_clear_ex_release_mutex(MVMThreadContext *tc);
+
+/* UV always defines this for Win32 but not Unix. Which is fine, as we can probe
+ * for it on Unix more easily than on Win32. */
+#if !defined(MVM_THREAD_LOCAL) && defined(UV_THREAD_LOCAL)
+#define MVM_THREAD_LOCAL UV_THREAD_LOCAL
+#endif
+
+#ifdef MVM_THREAD_LOCAL
+
+extern MVM_THREAD_LOCAL MVMThreadContext *MVM_running_threads_context;
+
+MVM_STATIC_INLINE MVMThreadContext *MVM_get_running_threads_context(void) {
+    return MVM_running_threads_context;
+}
+
+MVM_STATIC_INLINE void MVM_set_running_threads_context(MVMThreadContext *tc) {
+    MVM_running_threads_context = tc;
+}
+
+#else
+
+/* Fallback to an implememtation using UV's APIs (pretty much pthreads) */
+extern uv_key_t MVM_running_threads_context_key;
+
+MVM_STATIC_INLINE MVMThreadContext *MVM_get_running_threads_context(void) {
+    return uv_key_get(&MVM_running_threads_context_key);
+}
+
+MVM_STATIC_INLINE void MVM_set_running_threads_context(MVMThreadContext *tc) {
+    uv_key_set(&MVM_running_threads_context_key, tc);
+}
+
+#endif
