@@ -1,19 +1,42 @@
 /* Per-thread profiling data. */
 struct MVMProfileThreadData {
-    /* The root of the call graph. */
-    MVMProfileCallNode *call_graph;
-
     /* The current call graph node we're in. */
     MVMProfileCallNode *current_call;
 
+    /* Cache whether moar currently has confprogs enabled */
+    MVMuint8 is_dynamic_confprog_installed;
+    MVMuint8 is_static_confprog_installed;
+
+    /* The thread ID of the thread responsible for spawning this thread. */
+    MVMuint32 parent_thread_id;
+
+    /* The root of the call graph. */
+    MVMProfileCallNode *call_graph;
+
     /* The time we started profiling. */
     MVMuint64 start_time;
 
     /* The time we finished profiling, if we got there already. */
     MVMuint64 end_time;
 
-    /* The thread ID of the thread responsible for spawning this thread. */
-    MVMuint32 parent_thread_id;
+    /* The next two arrays are there to remove the need for the GC
+     * to walk the entire call graph every time.
+     */
+
+    /* Replaces static frame pointers in call nodes with an index into
+     * this array */
+    MVM_VECTOR_DECL(MVMStaticFrame *, staticframe_array);
+
+    /* Replaces type pointers in call nodes allocation entries with an
+     * index into this array */
+    MVM_VECTOR_DECL(MVMObject *, type_array);
+
+    /* When a confprog is deciding what frames to enter, exiting a frame
+     * when there's no node on the call stack can either be a run-time error,
+     * or it can be completely normal. To differentiate, we have to count the
+     * number of times we entered a frame without setting up a call graph node.
+     */
+    MVMuint32 non_calltree_depth;
 
     /* Garbage collection time measurements. */
     MVMProfileGC *gcs;
@@ -72,20 +95,21 @@ struct MVMProfileGC {
 
     /* Inter-generation links count */
     MVMuint32 num_gen2roots;
+
+    /* sum of num_gen2roots of all TCs that had work stolen by
+     * this thread */
+    MVMuint32 num_stolen_gen2roots;
+
+    MVMProfileDeallocationCount *deallocs;
+    MVMuint32 num_dealloc;
+    MVMuint32 alloc_dealloc; /* haha */
 };
 
 /* Call graph node, which is kept per thread. */
 struct MVMProfileCallNode {
     /* The frame this data is for.
      * If this CallNode is for a native call, this is NULL. */
-    MVMStaticFrame *sf;
-
-    /* If the static frame is NULL, we're collecting data on a native call */
-    char *native_target_name;
-
-    /* When was this node first entered, ever? */
-    MVMuint64 first_entry_time;
-
+    MVMuint32 sf_idx;
     /* The timestamp when we entered the node. */
     MVMuint64 cur_entry_time;
 
@@ -93,6 +117,9 @@ struct MVMProfileCallNode {
      * suspended due to GC or spesh. */
     MVMuint64 cur_skip_time;
 
+    /* Entry mode, persisted for the sake of continuations. */
+    MVMuint64 entry_mode;
+
     /* The node in the profiling call graph that we came from. */
     MVMProfileCallNode *pred;
 
@@ -133,14 +160,18 @@ struct MVMProfileCallNode {
     /* Number of times deopt_all happened. */
     MVMuint64 deopt_all_count;
 
-    /* Entry mode, persisted for the sake of continuations. */
-    MVMuint64 entry_mode;
+    /* If the static frame is NULL, we're collecting data on a native call */
+    char *native_target_name;
+
+    /* When was this node first entered, ever? */
+    MVMuint64 first_entry_time;
+
 };
 
 /* Allocation counts for a call node. */
 struct MVMProfileAllocationCount {
     /* The type we're counting allocations of. */
-    MVMObject *type;
+    MVMuint32 type_idx;
 
     /* The number of allocations we've counted. */
     /* a) in regularly interpreted code */
@@ -151,6 +182,24 @@ struct MVMProfileAllocationCount {
 
     /* c) in jitted code */
     MVMuint64 allocations_jit;
+
+    /* The number of allocations elimianted thanks to scalar replacement. */
+    MVMuint64 scalar_replaced;
+};
+
+struct MVMProfileDeallocationCount {
+    MVMObject *type;
+
+    /* How often was this type freed from the nursery with
+     * the "seen in nursery" flag not set? */
+    MVMuint32 deallocs_nursery_fresh;
+
+    /* How often was this type freed from the nursery with
+     * the "seen in nursery" flag set? */
+    MVMuint32 deallocs_nursery_seen;
+
+    /* How often was this type freed in the old generation? */
+    MVMuint32 deallocs_gen2;
 };
 
 /* When a continuation is taken, we attach one of these to it. It carries the
@@ -182,8 +231,11 @@ MVMProfileContinuationData * MVM_profile_log_continuation_control(MVMThreadConte
 void MVM_profile_log_continuation_invoke(MVMThreadContext *tc, const MVMProfileContinuationData *cd);
 void MVM_profile_log_thread_created(MVMThreadContext *tc, MVMThreadContext *child_tc);
 void MVM_profile_log_allocated(MVMThreadContext *tc, MVMObject *obj);
+void MVM_profile_log_scalar_replaced(MVMThreadContext *tc, MVMSTable *st);
 void MVM_profiler_log_gc_start(MVMThreadContext *tc, MVMuint32 full, MVMuint32 this_thread_responsible);
 void MVM_profiler_log_gc_end(MVMThreadContext *tc);
+void MVM_profiler_log_gen2_roots(MVMThreadContext *tc, MVMuint64 amount, MVMThreadContext *other);
+void MVM_profiler_log_gc_deallocate(MVMThreadContext *tc, MVMObject *object);
 void MVM_profiler_log_unmanaged_data_promoted(MVMThreadContext *tc, MVMuint64 amount);
 void MVM_profiler_log_spesh_start(MVMThreadContext *tc);
 void MVM_profiler_log_spesh_end(MVMThreadContext *tc);
