@@ -16,8 +16,8 @@ struct MVMSpeshLogGuard {
  * thread allows it to write more (effectively, the limit on the number of
  * outstanding work per thread). Threads other than the main one getting a
  * bit less buffer space helps reduce memory use a bit. */
-#define MVM_SPESH_LOG_QUOTA_MAIN_THREAD 2
-#define MVM_SPESH_LOG_QUOTA 1
+#define MVM_SPESH_LOG_QUOTA_MAIN_THREAD 3
+#define MVM_SPESH_LOG_QUOTA 2
 
 /* The number of logged invocations before we decide we've enough data for
  * the time being; should be at least the maximum threshold value in
@@ -47,6 +47,6 @@ void MVM_spesh_log_decont(MVMThreadContext *tc, MVMuint8 *prev_op, MVMObject *va
 void MVM_spesh_log_invoke_target(MVMThreadContext *tc, MVMObject *invoke_target,
     MVMuint16 was_multi);
 void MVM_spesh_log_return_type(MVMThreadContext *tc, MVMObject *value);
-void MVM_spesh_log_return_type_from_jit(MVMThreadContext *tc, MVMObject *value);
+void MVM_spesh_log_return_to_unlogged(MVMThreadContext *tc);
 void MVM_spesh_log_plugin_resolution(MVMThreadContext *tc, MVMuint32 bytecode_offset,
         MVMuint16 guard_index);
