@@ -12,6 +12,7 @@ typedef struct MVMAsyncTaskOps MVMAsyncTaskOps;
 typedef struct MVMAttributeIdentifier MVMAttributeIdentifier;
 typedef struct MVMBoolificationSpec MVMBoolificationSpec;
 typedef struct MVMBootTypes MVMBootTypes;
+typedef struct MVMEventSubscriptions MVMEventSubscriptions;
 typedef struct MVMBytecodeAnnotation MVMBytecodeAnnotation;
 typedef struct MVMCallCapture MVMCallCapture;
 typedef struct MVMCallCaptureBody MVMCallCaptureBody;
@@ -72,8 +73,6 @@ typedef struct MVMKnowHOWAttributeREPR MVMKnowHOWAttributeREPR;
 typedef struct MVMKnowHOWAttributeREPRBody MVMKnowHOWAttributeREPRBody;
 typedef struct MVMKnowHOWREPR MVMKnowHOWREPR;
 typedef struct MVMKnowHOWREPRBody MVMKnowHOWREPRBody;
-typedef struct MVMLexicalRegistry MVMLexicalRegistry;
-typedef struct MVMLoadedCompUnitName MVMLoadedCompUnitName;
 typedef struct MVMNFA MVMNFA;
 typedef struct MVMNFABody MVMNFABody;
 typedef struct MVMNFAStateInfo MVMNFAStateInfo;
@@ -126,7 +125,6 @@ typedef struct MVMConcBlockingQueue MVMConcBlockingQueue;
 typedef struct MVMConcBlockingQueueBody MVMConcBlockingQueueBody;
 typedef struct MVMConcBlockingQueueNode MVMConcBlockingQueueNode;
 typedef struct MVMObject MVMObject;
-typedef struct MVMObjectId MVMObjectId;
 typedef struct MVMObjectStooge MVMObjectStooge;
 typedef struct MVMOpInfo MVMOpInfo;
 typedef struct MVMOSHandle MVMOSHandle;
@@ -147,7 +145,6 @@ typedef struct MVMP6opaqueREPRData MVMP6opaqueREPRData;
 typedef struct MVMP6str MVMP6str;
 typedef struct MVMP6strBody MVMP6strBody;
 typedef union  MVMRegister MVMRegister;
-typedef struct MVMReprRegistry MVMReprRegistry;
 typedef struct MVMREPROps MVMREPROps;
 typedef struct MVMREPROps_Associative MVMREPROps_Associative;
 typedef struct MVMREPROps_Attribute MVMREPROps_Attribute;
@@ -169,6 +166,7 @@ typedef struct MVMSpeshAnn MVMSpeshAnn;
 typedef struct MVMSpeshFacts MVMSpeshFacts;
 typedef struct MVMSpeshCode MVMSpeshCode;
 typedef struct MVMSpeshCandidate MVMSpeshCandidate;
+typedef struct MVMSpeshCandidateBody MVMSpeshCandidateBody;
 typedef struct MVMSpeshLogGuard MVMSpeshLogGuard;
 typedef struct MVMSpeshCallInfo MVMSpeshCallInfo;
 typedef struct MVMSpeshInline MVMSpeshInline;
@@ -177,6 +175,7 @@ typedef struct MVMSpeshLog MVMSpeshLog;
 typedef struct MVMSpeshLogBody MVMSpeshLogBody;
 typedef struct MVMSpeshLogEntry MVMSpeshLogEntry;
 typedef struct MVMSpeshPluginState MVMSpeshPluginState;
+typedef struct MVMSpeshPluginStateBody MVMSpeshPluginStateBody;
 typedef struct MVMSpeshPluginPosition MVMSpeshPluginPosition;
 typedef struct MVMSpeshPluginGuardSet MVMSpeshPluginGuardSet;
 typedef struct MVMSpeshPluginGuard MVMSpeshPluginGuard;
@@ -201,10 +200,17 @@ typedef struct MVMSpeshUsages MVMSpeshUsages;
 typedef struct MVMSpeshUseChainEntry MVMSpeshUseChainEntry;
 typedef struct MVMSpeshDeoptUseEntry MVMSpeshDeoptUseEntry;
 typedef struct MVMSpeshFrameWalker MVMSpeshFrameWalker;
+typedef struct MVMSpeshPEAInfo MVMSpeshPEAInfo;
+typedef struct MVMSpeshPEAAllocation MVMSpeshPEAAllocation;
+typedef struct MVMSpeshPEADeopt MVMSpeshPEADeopt;
+typedef struct MVMSpeshPEAMaterializeInfo MVMSpeshPEAMaterializeInfo;
+typedef struct MVMSpeshPEADeoptPoint MVMSpeshPEADeoptPoint;
+typedef struct MVMConfigurationProgram MVMConfigurationProgram;
 typedef struct MVMSTable MVMSTable;
 typedef struct MVMStaticFrame MVMStaticFrame;
 typedef struct MVMStaticFrameBody MVMStaticFrameBody;
 typedef struct MVMStaticFrameInstrumentation MVMStaticFrameInstrumentation;
+typedef struct MVMStaticFrameDebugLocal MVMStaticFrameDebugLocal;
 typedef struct MVMStaticFrameSpesh MVMStaticFrameSpesh;
 typedef struct MVMStaticFrameSpeshBody MVMStaticFrameSpeshBody;
 typedef struct MVMStorageSpec MVMStorageSpec;
@@ -218,8 +224,6 @@ typedef struct MVMThread MVMThread;
 typedef struct MVMThreadBody MVMThreadBody;
 typedef struct MVMThreadContext MVMThreadContext;
 typedef struct MVMUnicodeNamedValue MVMUnicodeNamedValue;
-typedef struct MVMUnicodeNameRegistry MVMUnicodeNameRegistry;
-typedef struct MVMUnicodeGraphemeNameRegistry MVMUnicodeGraphemeNameRegistry;
 typedef struct MVMUninstantiable MVMUninstantiable;
 typedef struct MVMWorkThread MVMWorkThread;
 typedef struct MVMIOOps MVMIOOps;
@@ -268,10 +272,13 @@ typedef struct MVMProfileThreadData MVMProfileThreadData;
 typedef struct MVMProfileGC MVMProfileGC;
 typedef struct MVMProfileCallNode MVMProfileCallNode;
 typedef struct MVMProfileAllocationCount MVMProfileAllocationCount;
+typedef struct MVMProfileDeallocationCount MVMProfileDeallocationCount;
 typedef struct MVMProfileContinuationData MVMProfileContinuationData;
 typedef struct MVMHeapSnapshotCollection MVMHeapSnapshotCollection;
 typedef struct MVMHeapDumpIndex MVMHeapDumpIndex;
 typedef struct MVMHeapDumpIndexSnapshotEntry MVMHeapDumpIndexSnapshotEntry;
+typedef struct MVMHeapDumpTableOfContents MVMHeapDumpTableOfContents;
+typedef struct MVMHeapSnapshotStats MVMHeapSnapshotStats;
 typedef struct MVMHeapSnapshot MVMHeapSnapshot;
 typedef struct MVMHeapSnapshotType MVMHeapSnapshotType;
 typedef struct MVMHeapSnapshotStaticFrame MVMHeapSnapshotStaticFrame;
@@ -279,11 +286,17 @@ typedef struct MVMHeapSnapshotCollectable MVMHeapSnapshotCollectable;
 typedef struct MVMHeapSnapshotReference MVMHeapSnapshotReference;
 typedef struct MVMHeapSnapshotState MVMHeapSnapshotState;
 typedef struct MVMHeapSnapshotWorkItem MVMHeapSnapshotWorkItem;
-typedef struct MVMHeapSnapshotSeen MVMHeapSnapshotSeen;
 typedef struct MVMDebugServerBreakpointInfo MVMDebugServerBreakpointInfo;
 typedef struct MVMDebugServerBreakpointFileTable MVMDebugServerBreakpointFileTable;
 typedef struct MVMDebugServerBreakpointTable MVMDebugServerBreakpointTable;
 typedef struct MVMDebugServerHandleTable MVMDebugServerHandleTable;
 typedef struct MVMDebugServerHandleTableEntry MVMDebugServerHandleTableEntry;
+typedef struct MVMDebugServerRequestData MVMDebugServerRequestData;
 typedef struct MVMDebugServerRequestEntry MVMDebugServerRequestEntry;
 typedef struct MVMDebugServerData MVMDebugServerData;
+typedef struct MVMRunloopState MVMRunloopState;
+typedef struct MVMStrHashTable MVMStrHashTable;
+typedef struct MVMFixKeyHashTable MVMFixKeyHashTable;
+typedef struct MVMIndexHashTable MVMIndexHashTable;
+typedef struct MVMPtrHashTable MVMPtrHashTable;
+typedef struct MVMUniHashTable MVMUniHashTable;
